package cn.kdan.compdf.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.json.JSONUtil;
import cn.kdan.compdf.dto.AddNewProjectDTO;
import cn.kdan.compdf.dto.EditProjectDTO;
import cn.kdan.compdf.entity.BackgroundUserProject;
import cn.kdan.compdf.enums.APIKeysEnum;
import cn.kdan.compdf.enums.DateTImeFormatEnum;
import cn.kdan.compdf.enums.RabbitMQEnum;
import cn.kdan.compdf.exception.BusinessException;
import cn.kdan.compdf.mapper.BackgroundUserProjectMapper;
import cn.kdan.compdf.service.BackgroundUserProjectService;
import cn.kdan.compdf.service.BackgroundUserService;
import cn.kdan.compdf.utils.MD5Util;
import cn.kdan.compdf.utils.MyLocalDateUtil;
import cn.kdan.compdf.vo.BackgroundUserProjectVO;
import cn.kdan.compdf.vo.ProjectQueryVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author comPDF-Kit WPH 2022-10-09
 */
@Service
public class BackgroundUserProjectServiceImpl extends ServiceImpl<BackgroundUserProjectMapper, BackgroundUserProject> implements BackgroundUserProjectService {

    @Autowired
    private BackgroundUserService backgroundUserService;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Override
    public IPage<BackgroundUserProject> page(BackgroundUserProject query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }

    /**
     * 获取API Keys列表
     * @param userId 用户id
     * @return project列表
     */
    @Override
    public List<BackgroundUserProjectVO> getProjectList(Long userId,Integer timeZone) {
        // 查找用户下面状态正常的project记录
        LambdaQueryWrapper<BackgroundUserProject> eq = new LambdaQueryWrapper<BackgroundUserProject>()
                .eq(BackgroundUserProject::getStatus, 1)
                .eq(BackgroundUserProject::getUserId, userId)
                .orderByDesc(BackgroundUserProject::getCreateDate);
        List<BackgroundUserProject> list = this.baseMapper.selectList(eq);
        // 处理返回
        if (CollectionUtil.isNotEmpty(list)) {
            List<BackgroundUserProjectVO> backgroundUserProjectVOList = Lists.newArrayList();
            list.forEach(c -> {
                BackgroundUserProjectVO backgroundUserProjectVO = new BackgroundUserProjectVO();
                BeanUtil.copyProperties(c, backgroundUserProjectVO);
                backgroundUserProjectVO.setCreateDate(DateUtil.format(MyLocalDateUtil.timeZoneConversion(c.getCreateDate(),timeZone), DateTImeFormatEnum.DATE_TIME));
                backgroundUserProjectVOList.add(backgroundUserProjectVO);
            });
            return backgroundUserProjectVOList;
        } else {
            return Collections.emptyList();
        }
    }

    /**
     * 添加新项目
     * @param addNewProjectDTO 新项目参数
     */
    @Override
    public void addNewProject(AddNewProjectDTO addNewProjectDTO) {
        // 验证名称是否重复
        LambdaQueryWrapper<BackgroundUserProject> eq = new LambdaQueryWrapper<BackgroundUserProject>()
                .eq(BackgroundUserProject::getStatus, 1)
                .eq(BackgroundUserProject::getUserId, addNewProjectDTO.getUserId())
                .eq(BackgroundUserProject::getProjectName, addNewProjectDTO.getProjectName());
        Long selectCount = baseMapper.selectCount(eq);
        if (selectCount > 0) {
            throw new BusinessException("A project with the same name already exists");
        }
        BackgroundUserProject backgroundUserProject = new BackgroundUserProject();
        BeanUtil.copyProperties(addNewProjectDTO, backgroundUserProject);
        backgroundUserProject.setTenantId(addNewProjectDTO.getTenantId());
        backgroundUserProject.setProjectKey(APIKeysEnum.PUBLIC_KEY + MD5Util.MD5Encode(UUID.randomUUID().toString()));
        backgroundUserProject.setSecretKey(APIKeysEnum.SECRET_KEY + MD5Util.MD5Encode(UUID.randomUUID().toString()));
        backgroundUserProject.setStatus(1);
        this.baseMapper.insert(backgroundUserProject);
        // 发送消息同步SaaS服务
        String jsonStr = JSONUtil.toJsonStr(backgroundUserProject);
        rabbitTemplate.convertAndSend(RabbitMQEnum.BACKGROUND_EXCHANGE, RabbitMQEnum.BACKGROUND_API_ADD_ROUTING_KEY, jsonStr);
    }

    /**
     * 添加新用户默认项目
     * @param userId 用户id
     */
    @Override
    public void defaultProject(Long userId,Long tenantId) {
        // 验证是否已生成默认项目
        LambdaQueryWrapper<BackgroundUserProject> eq = new LambdaQueryWrapper<BackgroundUserProject>()
                .eq(BackgroundUserProject::getStatus, 1)
                .eq(BackgroundUserProject::getUserId, userId)
                .eq(BackgroundUserProject::getProjectName, APIKeysEnum.DEFAULT_PROJECT);
        Long selectCount = baseMapper.selectCount(eq);
        // 已生成退出
        if (selectCount > 0) {
            return;
        }
        // 新增
        BackgroundUserProject backgroundUserProject = new BackgroundUserProject();
        backgroundUserProject.setUserId(userId);
        backgroundUserProject.setTenantId(tenantId);
        backgroundUserProject.setProjectName(APIKeysEnum.DEFAULT_PROJECT);
        backgroundUserProject.setProjectKey(APIKeysEnum.PUBLIC_KEY + MD5Util.MD5Encode(UUID.randomUUID().toString()));
        backgroundUserProject.setSecretKey(APIKeysEnum.SECRET_KEY + MD5Util.MD5Encode(UUID.randomUUID().toString()));
        backgroundUserProject.setStatus(1);
        this.baseMapper.insert(backgroundUserProject);
        // 发送消息同步SaaS服务
        String jsonStr = JSONUtil.toJsonStr(backgroundUserProject);
        rabbitTemplate.convertAndSend(RabbitMQEnum.BACKGROUND_EXCHANGE, RabbitMQEnum.BACKGROUND_API_ADD_ROUTING_KEY, jsonStr);
    }

    /**
     * 编辑项目
     * @param editProjectDTO 编辑参数
     */
    @Override
    public void editProject(EditProjectDTO editProjectDTO) {
        // 验证名称是否重复
        LambdaQueryWrapper<BackgroundUserProject> eq = new LambdaQueryWrapper<BackgroundUserProject>()
                .ne(BackgroundUserProject::getId, editProjectDTO.getId())
                .eq(BackgroundUserProject::getStatus, 1)
                .eq(BackgroundUserProject::getProjectName, editProjectDTO.getProjectName());
        Long selectCount = baseMapper.selectCount(eq);
        if (selectCount > 0) {
            throw new BusinessException("A project with the same name already exists");
        }
        BackgroundUserProject backgroundUserProject = new BackgroundUserProject();
        BeanUtil.copyProperties(editProjectDTO, backgroundUserProject);
        this.baseMapper.updateById(backgroundUserProject);
    }

    /**
     * 删除项目
     * @param id 项目表主键
     */
    @Override
    public void deleteById(Long id) {
        // 通过id逻辑删除
        BackgroundUserProject backgroundUserProject = new BackgroundUserProject();
        backgroundUserProject.setId(id);
        backgroundUserProject.setStatus(0);
        this.baseMapper.updateById(backgroundUserProject);
        // 发送消息同步SaaS服务
        rabbitTemplate.convertAndSend(RabbitMQEnum.BACKGROUND_EXCHANGE, RabbitMQEnum.BACKGROUND_API_DEL_ROUTING_KEY, backgroundUserProject.getId());
    }

    @Override
    public List<ProjectQueryVO> getProjectQuery(Long tenantId){
        List<BackgroundUserProject> projects = this.baseMapper.selectList(new LambdaQueryWrapper<BackgroundUserProject>()
                .select(BackgroundUserProject::getId, BackgroundUserProject::getProjectName)
                .eq(BackgroundUserProject::getTenantId, tenantId)
                .eq(BackgroundUserProject::getStatus, 1)
                .orderByDesc(BackgroundUserProject::getId));
        List<ProjectQueryVO> projectQueryVOList = new ArrayList<>();
        projects.forEach(project -> {
            projectQueryVOList.add(new ProjectQueryVO(project.getId(),project.getProjectName()));
        });
        return projectQueryVOList;
    }

}
